@@ -8,7 +8,9 @@
 #include <limits>
 #include <algorithm>
 #include <iterator>
-#include <boost/pool/object_pool.hpp>
+#include "shared_ref.h"
+#include "slice.h"
+
 #define DEFAULT_LIST_CAPACITY 16
 
 namespace  pythonic {
@@ -20,21 +22,13 @@ namespace  pythonic {
         struct empty_list;
         template <class T> class list;
 
-
-        struct slice {
-            long lower, upper, step;
-            slice(long lower=0, long upper=std::numeric_limits<long>::max(), long step=1)
-                : lower(lower), upper(upper), step(step) {}
-        };
-
         template<class T>
             class list_view {
 
                 // data holder
                 typedef  typename std::remove_cv< typename std::remove_reference<T>::type>::type  _type;
                 typedef std::vector< _type > container_type;
-                size_t* refcount;
-                container_type* data; 
+                impl::shared_ref<container_type> data; 
 
                 template<class U>
                     friend class list;
@@ -57,19 +51,12 @@ namespace  pythonic {
                 typedef typename container_type::const_reverse_iterator const_reverse_iterator;
 
                 // constructor
-                list_view(): data(nullptr),refcount(nullptr) {}
-                list_view(list_view<T> const & s): data(s.data),refcount(s.refcount), slicing(s.slicing) { 
-                    if(refcount)
-                        ++*refcount;
-                }
+                list_view(): data(impl::no_memory()) {}
+                list_view(list_view<T> const & s): data(s.data), slicing(s.slicing) {}
                 list_view(list<T> & , slice const &);
-                ~list_view() {
-                    if(refcount) {
-                        assert(*refcount>0);
-                        --*refcount;
-                        //sg ! maybe leak
-                    }
-                }
+
+                // const getter
+                container_type const & get_data() const { return *data; }
 
                 // assignment
                 list_view& operator=(list<T> const & );
@@ -85,6 +72,25 @@ namespace  pythonic {
                 // size
                 size_type size() const { assert(slicing.step==1); return slicing.upper - slicing.lower ; }
 
+                // accessor
+                T const & operator[](long i) const { return (*data)[slicing.lower + i*slicing.step];}
+                T & operator[](long i) { return (*data)[slicing.lower + i*slicing.step];}
+
+                // comparison
+                template <class K>
+                    bool operator==(core::list<K> const & other) const {
+                        auto self_iter=begin();
+                        for(auto const & other_iter : other) {
+                            if(self_iter == end() ) return false;
+                            else if(*self_iter != other_iter) return false;
+                            ++self_iter;
+                        }
+                        return self_iter == end();
+                    }
+
+
+
+
             };
 
         /* the container type */
@@ -94,14 +100,7 @@ namespace  pythonic {
                 // data holder
                 typedef  typename std::remove_cv< typename std::remove_reference<T>::type>::type  _type;
                 typedef std::vector< _type > container_type;
-                size_t* refcount;
-                container_type* data; 
-
-                struct memory_size { size_t refcount; container_type data; };
-
-
-                template<class U>
-                    friend struct pythonic::_id;
+                impl::shared_ref<container_type> data; 
 
                 template<class U>
                     friend class list_view;
@@ -109,8 +108,6 @@ namespace  pythonic {
                 template<class U>
                     friend class list;
 
-                static boost::object_pool<memory_size> pool;
-
                 public:
 
                 // types
@@ -129,69 +126,57 @@ namespace  pythonic {
 
 
                 // constructors
-                list() :
-                    refcount(reinterpret_cast<size_t*>(pool.malloc())), data(new (refcount+1) container_type()) { *refcount=1; }
+                list() : data(impl::no_memory()) {}
                 template<class InputIterator>
-                    list(InputIterator start, InputIterator stop) :
-                        refcount(reinterpret_cast<size_t*>(pool.malloc())), data(new (refcount+1) container_type()) {
-                            data->reserve(DEFAULT_LIST_CAPACITY);
-                            std::copy(start, stop, std::back_inserter(*data));
-                            *refcount=1;
-                        }
-                list(empty_list const &) :
-                    refcount(reinterpret_cast<size_t*>(pool.malloc())), data(new (refcount+1) container_type()) { *refcount=1; }
-                list(size_type sz) :
-                    refcount(reinterpret_cast<size_t*>(pool.malloc())), data(new (refcount+1) container_type(sz)) { *refcount=1; }
-                list(std::initializer_list<value_type> l) :
-                    refcount(reinterpret_cast<size_t*>(pool.malloc())), data(new (refcount+1) container_type(l)) { *refcount=1; }
-                list(list<T> const & other) :
-                    data(const_cast<list<T>*>(&other)->data), refcount(const_cast<list<T>*>(&other)->refcount) { ++*refcount; }
-                template<class F>
-                    list(list<F> const & other) :
-                        refcount(reinterpret_cast<size_t*>(pool.malloc())), data(new (refcount+1) container_type(other.data->size())) {
-                            *refcount=1; 
-                            std::copy(other.begin(), other.end(), begin());
-                        }
-                ~list() {
-                    assert(*refcount>0);
-                    if(not --*refcount) { pool.free( reinterpret_cast<memory_size*>(refcount)); }
-                }
-
-                list<T>& operator=(list<T> const & other) {
-                    assert(*refcount>0);
-                    if(other.data != data) {
-                        if(not --*refcount) {  pool.free( reinterpret_cast<memory_size*>(refcount) ); }
-                        data=const_cast<list<T>*>(&other)->data;
-                        refcount=const_cast<list<T>*>(&other)->refcount;
-                        ++*refcount;
+                    list(InputIterator start, InputIterator stop) : data() {
+                        data->reserve(DEFAULT_LIST_CAPACITY);
+                        std::copy(start, stop, std::back_inserter(*data));
                     }
+                list(empty_list const &) : data() {}
+                list(size_type sz) :data(sz) {}
+                list(std::initializer_list<value_type> l) : data(std::move(l)) {}
+                list(list<T> && other) : data(std::move(other.data)) {}
+                list(list<T> const & other) : data(other.data) {}
+                template<class F>
+                    list(list<F> const & other) : data(other.size()) {
+                        std::copy(other.begin(), other.end(), begin());
+                    }
+                list(list_view<T> const & other) : data( other.begin(), other.end()) {}
+
+                list<T>& operator=(list<T> && other) {
+                    data=std::move(other.data);
                     return *this;
                 }
+                list<T>& operator=(list<T> const & other) {
+                    data=other.data;
+                    return *this;
+                }
+                list<T>& operator=(empty_list const & ) {
+                    data=impl::shared_ref<container_type>();
+                    return *this;
+                }
+
                 list<T>& operator=(list_view<T> const & other) {
-                    assert(*refcount >0) ;
                     if(other.data == data ) {
                         auto it = std::copy(other.begin(), other.end(), data->begin());
                         data->resize(it - data->begin());
                     }
                     else {
-                        if(not --*refcount) {
-                            /* should free the memory then reallocate it, so do nothing */
-                            /* HERE */
-                        }
-                        else {
-                            refcount=reinterpret_cast<size_t*>(pool.malloc());
-                            data=new (refcount+1) container_type(other.end() - other.begin());
-                        }
-                        *refcount=1;
-                        std::copy(other.begin(), other.end(), data->begin());
+                        data=impl::shared_ref<T>(other.begin(),other.end());
                     }
                     return *this;
                 }
                 list<T>& operator+=(list_view<T> const & other) {
-                    data->resize(data->size() + len(other));
+                    data->resize(data->size() + other.size());
                     std::copy(other.begin(), other.end(), data->begin());
                     return *this;
                 }
+                list<T> operator+(list_view<T> const & other) const {
+                    list<T> new_list(begin(), end());
+                    new_list.reserve(data->size() + other.size());
+                    std::copy(other.begin(), other.end(), std::back_inserter(new_list));
+                    return new_list;
+                }
 
                 // iterators
                 iterator begin() { return data->begin(); }
@@ -203,24 +188,30 @@ namespace  pythonic {
                 reverse_iterator rend() { return data->rend(); }
                 const_reverse_iterator rend() const { return data->rend(); }
 
-                // element access
-                reference operator[]( long n ) {
-#ifndef NDEBUG
-                    return (*data)[(n>=0)?n : (data->size() + n)];
-#else
-                    return (*data).at((n>=0)?n : (data->size() + n));
-#endif
-                }
-                const_reference operator[]( long n ) const {
-#ifndef NDEBUG
-                    return (*data)[(n>=0)?n : (data->size() + n)];
-#else
-                    return (*data).at((n>=0)?n : (data->size() + n));
-#endif
+                // comparison
+                int operator<(list<T> const& other) const {
+                    auto other_iter = other.begin();
+                    auto self_iter = begin();
+                    for(; other_iter != other.end() and self_iter != end(); ++other_iter, ++self_iter) {
+                        if( *other_iter < *self_iter ) return 1;
+                        if( *other_iter > *self_iter ) return -1;
+                    }
+                    if(other_iter !=other.end() and self_iter == end()) return 1;
+                    if(other_iter ==other.end() and self_iter != end()) return -1;
+                    return 0;
                 }
 
-                list<T> operator[]( slice const &s ) {
+                // element access
+                reference operator[]( long n ) {
+                    return (*data)[(n>=0)?n : (data->size() + n)];
+                }
+                const_reference operator[]( long n ) const {
+                    return (*data)[(n>=0)?n : (data->size() + n)];
+                }
+
+                list<T> operator[]( slice const &s ) const {
                     list<T> out(0);
+                    out.reserve(size());
                     long lower, upper;
                     if(s.step<0) {
                         if( s.lower == std::numeric_limits<long>::max() )
@@ -244,19 +235,43 @@ namespace  pythonic {
                     return out;
                 }
 
-                list_view<T> operator()( slice const &s) {
-                    return list_view<T>(*this, s);
+                list_view<T> operator()( slice const &s) const {
+                    return list_view<T>(*const_cast<list<T>*>(this), s); // SG: ugly !
                 }
 
                 // modifiers
-                void push_back( const T& x) { data->push_back(x); }
-                void insert(size_t i, const T& x) { data->insert(data->begin()+i, x); }
+                void push_back( T const & x) { data->push_back(x); }
+                void insert(size_t i, T const & x) { data->insert(data->begin()+i, x); }
                 void reserve(size_t n) { data->reserve(n); }
                 void resize(size_t n) { data->resize(n); }
                 iterator erase(size_t n) { return data->erase(data->begin()+n); }
 
+                // const getter
+                container_type const & get_data() const { return *data; }
+
+
+                T pop(long x = -1) {
+                    x = x%size();
+                    if (x<0) x+=size();
+                    T res = (*this)[x];
+                    erase(x);
+                    return res;
+                }
+
+                //TODO: have to raise a valueError
+                void remove(T const& x) {
+                    erase(index(x));
+                }
+
+
+                //Misc
+                //TODO: have to raise a valueError
+                long index(T const& x) {
+                    return std::find(begin(),end(),x)-begin();
+                }	
+
                 // list interface
-                operator bool() { return not data->empty(); }
+                operator bool() const { return not data->empty(); }
 
                 template <class F>
                     list<decltype(std::declval<T>()+std::declval<typename list<F>::value_type>())> operator+(list<F> const & s) const {
@@ -293,32 +308,25 @@ namespace  pythonic {
                     }
                 long size() const { return data->size(); }
 
-                T pop(long x = -1) {
-                    x = x%size();
-                    if (x<0) x+=size();
-                    T res = (*data)[x];
-                    erase(x);
-                    return res;
-                }   
 
 
             };
-        template<class T>
-            boost::object_pool<typename list<T>::memory_size> list<T>::pool;
 
 
         /* empty list implementation */
         struct empty_list {
             template<class T> // just for type inference, should never been instantiated
                 list<T> operator+(list<T> const & s) { return s; }
+            template<class T> // just for type inference, should never been instantiated
+                list_view<T> operator+(list_view<T> const & s) { return s; }
             empty_list operator+(empty_list const &) { return empty_list(); }
+            operator bool() { return false; }
         };
 
         /* list_view implementation */
         template<class T>
             list_view<T>::list_view(list<T> & other, slice const &s):
-                refcount(other.refcount), data(other.data), slicing(s) {
-                    ++*refcount;
+                data(other.data), slicing(s) {
                     long lower, upper;
                     if(slicing.step<0) {
                         if( slicing.lower == std::numeric_limits<long>::max() )
@@ -340,16 +348,13 @@ namespace  pythonic {
             list_view<T>& list_view<T>::operator=(list_view<T> const & s) {
                 slicing=s.slicing;
                 if(data != s.data) {
-                    refcount=s.refcount;
                     data=s.data;
-                    ++*refcount;
                 }
                 return *this;
             }
 
         template<class T>
             list_view<T>& list_view<T>::operator=(list<T> const & seq) {
-                list<T> out;
                 long lower = slicing.lower >= 0L ? slicing.lower : ( slicing.lower + data->size());
                 lower = std::max(0L,lower);
                 long upper = slicing.upper >= 0L ? slicing.upper : ( slicing.upper + data->size());
@@ -362,17 +367,18 @@ namespace  pythonic {
                 else {
                     assert("not implemented yet");
                 }
-                if( data!= seq.data ) ++*refcount;
                 return *this;
             }
         template<class T>
             list<T> list_view<T>::operator+(list_view<T> const & s) {
-                assert(s.data);
-                assert(data);
                 list<T> out(size() + s.size());
                 std::copy(s.begin(), s.end(), std::copy(begin(), end(), out.begin()));
                 return out;
             }
+        template<class N, class T>
+            list<T> operator*(N n, core::list<T> const& l) {
+                return l*n;
+            }
 
     }
 
