@@ -5,15 +5,15 @@
 #include "core/utils.h"
 #include "core/functional.h"
 #include "core/none.h"
+#include "core/string.h"
 #include "core/list.h"
+#include "core/exceptions.h"
+#include "core/iterator.h"
 #include "core/set.h"
 #include "core/dict.h"
-#include "core/string.h"
 #include "core/tuple.h"
-#include "core/intrinsics.h"
 #include "core/io.h"
-#include "core/exceptions.h"
-#include "core/generator.h"
+#include "core/intrinsics.h"
 
 /* additional modules */
 #include "modules/dispatch.h"
@@ -25,6 +25,9 @@
 #include "modules/cmath.h"
 #include "modules/random.h"
 #include "modules/iterator.h"
+#include "modules/string.h"
+#include "modules/time.h"
+#include "modules/omp.h"
 
 /* extensions */
 #include "core/parallel.h"
